"""
Framework integration callback handlers for automatic decision capture.

Provides drop-in callbacks for popular AI frameworks:
  - LangChain: BriefcaseLangChainHandler
  - LlamaIndex: BriefcaseLlamaIndexHandler
  - OpenAI Agents SDK: OpenAIAgentsTracer, openai_agents_hook
  - PageIndex: PageIndexTracer, PageIndexMCPObserver
"""

from briefcase.integrations.frameworks.langchain_handler import (
    BriefcaseLangChainHandler,
)
from briefcase.integrations.frameworks.llamaindex_handler import (
    BriefcaseLlamaIndexHandler,
)
from briefcase.integrations.frameworks import openai_agents_handler as openai_agents_hook
from briefcase.integrations.frameworks.openai_agents_handler import (
    OpenAIAgentsTracer,
)
from briefcase.integrations.frameworks.pageindex_handler import (
    PageIndexTracer,
)
from briefcase.integrations.frameworks.pageindex_mcp import (
    PageIndexMCPObserver,
)

__all__ = [
    "BriefcaseLangChainHandler",
    "BriefcaseLlamaIndexHandler",
    "OpenAIAgentsTracer",
    "openai_agents_hook",
    "PageIndexTracer",
    "PageIndexMCPObserver",
]
