"""Compatibility bridge for ``briefcase_ai.integrations``."""

from briefcase_ai._compat import _bridge_module

_bridge_module(
    __name__,
    "briefcase.integrations",
    submodules=("lakefs", "frameworks", "vcs"),
)
